// Game example
// A platform game, horizontal scrolling, with TOMMIO!
// NOTE: REQUIRES A LOCAL SERVER RUNNING, load with http://localhost:8000

// first, let's make a generative "landscape" with platforms
// each platform will be an object with an x, y, and w parameter

var platforms = [];
var marioY = 0;     // start mario character at top
var marioDy = 0;    // NEW: vertical velocity of mario character
var img;

/////// PLATFORM OBJECT CODE /////

function newPlatform(px, py, pw) {
    var p = {x: px, y: py, w: pw,
             right: platRight};
    return p;
}

// compute the location of the right end of a platform
function platRight() {
    return this.x + this.w;
}

///////////////////////////////////

function preload() {
    img = loadImage("http://localhost:8000/supertommio.png");
}

function setup() {
    createCanvas(600, 300);
    var pl = newPlatform(600, 200, 200);  // first platform
    platforms.push(pl);
}

function draw() {
    background("lightblue");
    fill("green");
    stroke("green");
    rect(0, height - 50, width, 50); // the ground
    
    fill(0);
    stroke(0);
    for (var i = 0; i < platforms.length; i++) {
        var p = platforms[i];
        p.x = p.x - 1;
        rect(p.x, p.y, p.w, 10);
    }

    // if first platform is offscreen to left, remove it
    if (platforms.length > 0 && platforms[0].right() < 0) {
        platforms.shift();
    }

    // if last platform is totally within canvas, make a new one
    var lastPlat = platforms[platforms.length-1];
    if (lastPlat.right() == width - 1) {
        var p = newPlatform(
              width, // left edge just off canvas
              floor(random(50, 225)), // height of new platform
              200);   // all platforms are 200 pixels wide (for now)
        platforms.push(p); // add to our array of platforms
    }

    // NEW CODE
    // move and draw the character "mario"
    // use linear search to find platform in the middle of canvas
    var i = 0;
    var marioX = width / 2;
    while (platforms[i].right() < marioX) {
        i += 1;
    }
    // now pindex is index of the platform in the middle of canvas
    // find the platform height
    var py = platforms[i].y;
    // show some debugging information for now
    text("Platform " + i + " py " + py + " Dy " + marioDy, 10, 50);

    // if we are above the platform, fall toward it, but do not go past it
    if (marioY <= py) {  // above or on the platform
        marioY = min(py, marioY + marioDy); // change Y by Dy
    } else { // below the platform
        if (marioDy < 0) {  // we don't want upward movement
            marioDy = 0;
        }
        marioY = min(height, marioY + marioDy);
    }
    if (marioY >= height) {
        marioY = 0;
        marioDy = 0;
    }

    // draw the "mario"
    fill("brown");
    stroke("brown");
    image(img, marioX, marioY - 75, 50, 75);

    //debugging:
    text(platforms.length.toString() + " platform(s)", 10, 30);

    // accelerate "mario" with gravity
    marioDy = marioDy + 1;
}

function keyPressed() {
    marioDy = -10;    // velocity set to up when key is pressed
}
